import os
import time
import datetime
import matplotlib  # noqa: F401
# Configuration file for the Sphinx documentation builder for
# matplotlib projects.

# Release mode enables optimizations and other related options.
is_release_build = tags.has('release')  # noqa

# Parse year using SOURCE_DATE_EPOCH, falling back to current time.
# https://reproducible-builds.org/specs/source-date-epoch/
build_date = datetime.datetime.utcfromtimestamp(
    int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
)

# -- Project information -----------------------------------------------------

project = "Matplotlib Sphinx Theme"
copyright = (
    f"2012 - {build_date.year} The Matplotlib development team"
)
author = "Matplotlib Developers"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'matplotlib.sphinxext.plot_directive',
]

# Add any paths that contain templates here, relative to this directory.

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# -- Options for HTML output -------------------------------------------------

html_theme = "mpl_sphinx_theme"
html_favicon = "_static/favicon.ico"
html_theme_options = {
    # logo is installed by mpl-sphinx-theme as:
    # "logo": {"link": "https://matplotlib.org/stable/",
    #         "image_light": "_static/logo_light.svg",
    #         "image_dark": "_static/logo_dark.svg"},
    # if this default is OK, then no need to modify "logo"
    # collapse_navigation in pydata-sphinx-theme is slow, so skipped for local
    # and CI builds https://github.com/pydata/pydata-sphinx-theme/pull/386
    "collapse_navigation": not is_release_build,
    "show_prev_next": False,
    # Determines the type of links produced in the navigation header:
    # - absolute: Links point to the URL https://matplotlib.org/...
    # - server-stable: Links point to top-level of the server /stable/...
    # - internal: Links point to the internal files as expanded by the `pathto`
    #   template function in Sphinx.
    "navbar_links": "absolute",
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]



from ._version import version_info, __version__  # noqa: F401

from pathlib import Path


def set_config_defaults(app):
    """Set default logo in theme options."""
    try:
        theme = app.builder.theme_options
    except AttributeError:
        theme = None
    if not theme:
        theme = {}

    # Default logo
    logo = theme.get("logo", {})
    if "image_dark" not in logo:
        logo["image_dark"] = "_static/logo_dark.svg"
    if "image_light" not in logo:
        logo["image_light"] = "_static/logo_light.svg"
    if "link" not in logo:
        logo["link"] = "https://matplotlib.org/stable/"
    theme["logo"] = logo

    # Update the HTML theme config
    app.builder.theme_options = theme


def get_html_theme_path():
    """Return list of HTML theme paths."""
    return [str(Path(__file__).parent.parent.resolve())]


def setup_html_page_context(app, pagename, templatename, context, doctree):
    """Add a mpl_path template function."""
    # Pick link setting based on release mode. Theme users may specify either:
    # 1. a single string indicating the mode for navbar links;
    # 2. a tuple of two strings, the first being the mode for development, and
    #    the second being the mode for release.
    # Release mode is determined by specifying the 'release' tag during build.
    navbar_links = context['theme_navbar_links']
    if not isinstance(navbar_links, tuple):
        # Allow a single string for backwards compatibility.
        navbar_links = (navbar_links, navbar_links)
    if (len(navbar_links) != 2 or
            any(opt not in ['internal', 'absolute', 'server-stable']
                for opt in navbar_links)):
        raise ValueError(f'Invalid navbar_links theme option: {navbar_links}')
    navbar_links = (navbar_links[1] if app.tags.has('release') else
                    navbar_links[0])

    def mpl_path(path):
        if navbar_links == 'internal':
            pathto = context['pathto']
            return pathto(path)
        elif navbar_links == 'absolute':
            return f'https://matplotlib.org/stable/{path}'
        elif navbar_links == 'server-stable':
            return f'/stable/{path}'
        else:
            raise ValueError(
                f'Invalid navbar_links theme option: {navbar_links}')
    context['mpl_path'] = mpl_path


# For more details, see:
# https://www.sphinx-doc.org/en/master/development/theming.html#distribute-your-theme-as-a-python-package
def setup(app):
    here = Path(__file__).parent.resolve()
    # Include component templates
    app.config.templates_path.append(str(here / "components"))
    app.add_html_theme("mpl_sphinx_theme", str(here))
    app.connect("builder-inited", set_config_defaults)
    app.connect("html-page-context", setup_html_page_context)
    return {'version': __version__, 'parallel_read_safe': True}
